
/**************    This simulation file runs 3D viscous cylindrical filament subjected to time periodic body force in Basilisk open source code(http://basilisk.fr).   The code is written in cartesian coordinate system. We have cubic domain with cylinder at the center of fluid1 and outside as fluid2.
The code solves two phase flow using VOF method. *********/




#include "navier-stokes/centered.h"
#include "navier-stokes/perfs.h"
#include "two-phase.h"
#include "tension.h"
#include "output_vtu_foreach.h"


#define FILTERED
# define SIGMA 20.7     						//Surface tension coefficient
#define epsilon 0.005    						// Inital perturbation amplitude a(0)
#define radius 0.2							//Radius of cylinder R0
double k = 4.8;							//Wavenumber along z
int m = 0;								//Wavenumber along thetha(azimuthal) direction.

# define domain_size (2*M_PI/k)*1				//Domain size of one wavelength
double omega = 600.0*M_PI;				// Forcing frequency
int grid_size = 32;
double amp = 1.8e04;						// Forcing amplitude




/*  Boundary conditions set to outflow on left right top bottom wall   */

u.n[left]  =neumann(0); 
u.t[left]  =neumann(0); 
u.r[left]  =neumann(0); 
p[left]    = dirichlet(0);

u.n[right]  =neumann(0); 
u.t[right]  =neumann(0); 
u.r[right]  =neumann(0); 
p[right]    = dirichlet(0);

u.n[top]  =neumann(0); 
u.t[top]  =neumann(0); 
u.r[top]  =neumann(0); 
p[top]    = dirichlet(0);

u.n[bottom]  =neumann(0); 
u.t[bottom]  =neumann(0); 
u.r[bottom]  =neumann(0); 
p[bottom]    = dirichlet(0);





int main (int argc, char * argv[])
{

  
  size (domain_size);  
  init_grid(grid_size);  
  periodic(front);  												//Periodic Boundary condition along length of cylinder
  origin (-domain_size/2.0,-domain_size/2.0,-domain_size/2.0);			//Shifting origin to center of domain
	
  /**
  We set the density and viscosity of each phase as well as the
  surface tension coefficient and start the simulation. */
  
  rho1 = 0.957, rho2 =0.001;
  mu1 = 0.1, mu2 = 0.001;  
  f.sigma = SIGMA;

  run();
}

/**
## Initial conditions */

event init (t = 0) {
	
	
    if (!restore (file ="restart")) {
	  
    refine (sq(x) + sq(y) < sq(radius+0.05) && level <9);							 //Inital refinement of uniform 512 grid refinement in cylinder of radius (R0+0.05) 
	
    fraction(f, sq(radius + epsilon*cos(k*z)*cos(m*atan2(x,y))) - sq(x) - sq(y)); 			 // Define perturbed cylinder 
	  

  }
  
}

/********* Time periodic body force term *******/ 

event acceleration (i++) {
	


  face vector av = a;
  foreach_face(x)
    av.x[] = -amp*sin(atan2(x,y))*cos(omega*t)*(sqrt((x*x + y*y))/radius);		
  foreach_face(y)
    av.y[] = -amp*cos(atan2(x,y))*cos(omega*t)*(sqrt((x*x + y*y))/radius);
	


}



/**
## Outputs

We log some statistics on the solver about time and time step */

event logfile (i++) {

if(pid()==0) { 
	 

 	FILE *file3;
	file3  = fopen("track.out","aw"); 

  fprintf (file3,"%.4e \t %.4e  \n",t, dt);
  fclose(file3);

  }
}




/**
We save snapshots of the simulation at regular intervals to
restart or to post-process */




event snapshot (t = 0.; t += 0.01; t <= 20.0) {

  char name[80];
  sprintf (name, "snapshot-%g", t);
  scalar pid[];
  foreach()
    pid[] = fmod(pid()*(npe() + 37), npe());
  boundary ({pid});
  dump (name);
  dump ("restart");

}




void backup_fields (scalar f, vector u, int nf)
{
  char name[80], subname[80];
  FILE * fp ;
	nf > 0 ? sprintf(name, "sol_%4.4d_n%3.3d.vtu", nf,pid()) : sprintf(name, "sol_n%3.3d.vtu",pid());
	fp = fopen(name, "w"); output_vtu_ascii_foreach ((scalar *) {f}, (vector *) {u}, N, fp, false); fclose (fp);

#if _MPI
	if (pid()==0){
		nf > 0 ? sprintf(name, "sol-%d.pvtu", nf) : sprintf(name, "sol-0.pvtu");
		nf > 0 ? sprintf(subname, "sol_%4.4d", nf) : sprintf(subname, "sol");
		fp = fopen(name, "w"); output_pvtu_ascii ((scalar *) {f}, (vector *) {u}, N, fp, subname); fclose (fp);
	}
	MPI_Barrier(MPI_COMM_WORLD);
#endif
	
}



// Output data //


event logfile2(t = 0.; t += 0.01; t <= 20.0) {
	
	backup_fields(f,u,t*100);
	
}


/*** Adaptive refinement based on field variables f,u with min and max level of refinement of 9  *********/


event adapt (i++) {

  adapt_wavelet ((scalar*){f,u},(double[]){0.01,0.01,0.01,0.01},9,9);
  unrefine ( (sq(x) + sq(y) > sq(radius+0.1) ) && level >= 4);
  
}





